import asyncio
import platform

import psutil

from prtg_pyprobe.sensors.helpers import (
    SensorData,
    SensorDefinitionGroup,
    SensorDefinition,
)
from prtg_pyprobe.sensors.sensor import SensorBase


class Sensor(SensorBase):
    @property
    def name(self) -> str:
        return "Probe Health"

    @property
    def kind(self) -> str:
        return "mpprobehealth"

    @property
    def definition(self) -> dict:
        probe_health_def = SensorDefinition(
            name=self.name,
            kind=self.kind,
            description="Internal sensor used to monitor the health of a PRTG probe on the system "
            "the mini probe is running on",
            default="yes",
            sensor_help="Internal sensor used to monitor the health of a PRTG probe on the system "
            "the mini probe is running on",
            tag="mpprobehealthsensor",
        )
        probe_health_def_group_temperature = SensorDefinitionGroup(name="temperature", caption="Temperature Settings")
        probe_health_def_group_temperature.add_field(
            field_type="radio",
            name="celfar",
            caption="Choose between Celsius or Fahrenheit display",
            help="Choose wether you want to return the value in Celsius or Fahrenheit",
            options={"1": "Celsius", "2": "Fahrenheit"},
            default="1",
        )
        probe_health_def_group_temperature.add_field(
            field_type="radio",
            name="diskfull",
            caption="Full Display of all disk space values",
            help="Choose wether you want to get all disk space data or only percentages.",
            options={"1": "Percentages", "2": "Full Information"},
            default="1",
        )
        probe_health_def.add_group(probe_health_def_group_temperature)
        return probe_health_def.data

    async def work(self, task_data: dict, q: asyncio.Queue) -> None:
        probe_health_data = SensorData(sensor_id=task_data["sensorid"])
        uname = platform.uname()
        probe_health_data.message = f"Running on {uname.system} in version {uname.version}! OK"
        cpu_load = psutil.getloadavg()
        vmemory = psutil.virtual_memory()
        swapmemory = psutil.swap_memory()
        disk_partitions = psutil.disk_partitions()
        for partition in disk_partitions:
            part_disk_usage = psutil.disk_usage(partition[1])
            probe_health_data.add_channel(
                name=f"Disk Space Percent {partition[1]}",
                mode="float",
                kind="Percent",
                value=part_disk_usage[3],
            )
            if task_data["diskfull"] == "2":
                probe_health_data.add_channel(
                    name=f"Disk Space Total {partition[1]}",
                    mode="integer",
                    kind="BytesDisk",
                    value=part_disk_usage[0],
                )

                probe_health_data.add_channel(
                    name=f"Disk Space Used {partition[1]}",
                    mode="integer",
                    kind="BytesDisk",
                    value=part_disk_usage[1],
                )
                probe_health_data.add_channel(
                    name=f"Disk Space Free {partition[1]}",
                    mode="integer",
                    kind="BytesDisk",
                    value=part_disk_usage[2],
                )

        probe_health_data.add_channel(name="Memory Total", mode="integer", kind="BytesMemory", value=vmemory.total)
        probe_health_data.add_channel(
            name="Memory Available",
            mode="integer",
            kind="BytesMemory",
            value=vmemory.available,
        )
        probe_health_data.add_channel(
            name="Swap Total",
            mode="integer",
            kind="BytesMemory",
            value=swapmemory.total,
        )
        probe_health_data.add_channel(name="Swap Free", mode="integer", kind="BytesMemory", value=swapmemory.free)
        probe_health_data.add_channel(
            name="Load average 1 min",
            mode="float",
            kind="Custom",
            customunit="",
            value=float(cpu_load[0]),
        )
        probe_health_data.add_channel(
            name="Load average 5 min",
            mode="float",
            kind="Custom",
            customunit="",
            value=float(cpu_load[1]),
        )
        probe_health_data.add_channel(
            name="Load average 10 min",
            mode="float",
            kind="Custom",
            customunit="",
            value=float(cpu_load[2]),
        )
        await q.put(probe_health_data.data)



import logging

from logging.handlers import TimedRotatingFileHandler


def setup_logging():
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)
    formatter = logging.Formatter("%(asctime)s :: %(levelname)s :: %(message)s")

    console_handler = logging.StreamHandler()
    console_handler.setLevel(level=logging.DEBUG)
    console_handler.setFormatter(formatter)
    logger.addHandler(console_handler)

    return logger, console_handler, formatter


def setup_file_logging(cfg, logger, formatter, console_handler):
    try:
        logger.setLevel(cfg["log_level"])

        if cfg["log_file_location"]:
            filehandler = TimedRotatingFileHandler(cfg["log_file_location"], when="d", interval=1, backupCount=7)
            filehandler.setFormatter(formatter)
            logger.addHandler(filehandler)
            logger.removeHandler(console_handler)
        return logger
    # todo: catch explicit
    except Exception:
        raise



import asyncio
import json
import time

from requests import Response

from testfixtures import LogCapture

import prtg_pyprobe.run
from prtg_pyprobe.run import main
from prtg_pyprobe.communication.communication_http_api import PRTGHTTPApi
import prtg_pyprobe.monitoring


def test_run(monkeypatch, mocker):
    monkeypatch.setenv(
        name="PROBE_CONFIG",
        value='{"disable_ssl_verification": false, "log_file_location": "", "log_level": "DEBUG", "probe_access_key": '
        '"miniprobe", "probe_access_key_hashed": "cd7b773e2ce4205e9f5907b157f3d26495c5b373", '
        '"probe_base_interval": "60", "probe_gid": "72C461B5-F768-470B-A1A8-2D5F5DEDDF8F", '
        '"probe_name": "Python Mini Probe UT", "probe_protocol_version": "1", '
        '"prtg_server_ip_dns": "ut.prtg,com", "prtg_server_port": "443"}',
    )
    config = {
        "disable_ssl_verification": False,
        "log_file_location": "",
        "log_level": "DEBUG",
        "probe_access_key": "miniprobe",
        "probe_access_key_hashed": "cd7b773e2ce4205e9f5907b157f3d26495c5b373",
        "probe_base_interval": "60",
        "probe_gid": "72C461B5-F768-470B-A1A8-2D5F5DEDDF8F",
        "probe_name": "Python Mini Probe UT",
        "probe_protocol_version": "1",
        "prtg_server_ip_dns": "ut.prtg,com",
        "prtg_server_port": "443",
    }
    monkeypatch.setattr(PRTGHTTPApi, "send_announce", return_response_200)
    monkeypatch.setattr(PRTGHTTPApi, "get_tasks", return_response_200)
    monkeypatch.setattr(PRTGHTTPApi, "send_data", return_response_200)
    monkeypatch.setattr(Response, "json", return_dict_json)
    monkeypatch.setattr(Response, "json", return_dict_json)
    monkeypatch.setattr(asyncio, "get_event_loop", mock_event_loop)
    monkeypatch.setattr(prtg_pyprobe.monitoring, "monitoring", mock_monitoring)
    monkeypatch.setattr(time, "sleep", return_true)
    with LogCapture() as log:
        main()
    log.check_present(
        ("root", "INFO", "Starting pyprobe!"),
        ("root", "INFO", "Loading configuration"),
        ("root", "INFO", "Config loaded and probe announcing to PRTG"),
        ("root", "DEBUG", f"Your current config: \n {json.dumps(config, indent=2)}"),
        ("root", "INFO", "Initializing SNMP Engine"),
        ("root", "INFO", "Initializing PRTG API"),
        ("root", "INFO", "Initializing Sensors"),
        ("root", "INFO", "Init done!"),
        ("root", "DEBUG", f"Tasks received from PRTG {return_dict_json()}"),
        ("root", "INFO", "Monitoring data successfully sent to PRTG Core."),
    )


def return_response_200(*args, **kwargs):
    resp = Response()
    resp.status_code = 200
    resp._content = (
        b'{"sensorid": "1234", "timeout": 5, "target": "https://example.com","request_type": "GET",'
        b'"acceptable_status_codes": "","headers": "","post_data": "",}'
    )
    return resp


def return_dict_json(*args, **kwargs):
    return [
        {
            "sensorid": "1234",
            "timeout": 5,
            "target": "https://example.com",
            "request_type": "GET",
            "acceptable_status_codes": "",
            "headers": "",
            "post_data": "",
        }
    ]


class MockEventLoop(object):
    @staticmethod
    def run_until_complete(self):
        return "work"

    @staticmethod
    def is_running():
        return False


def mock_event_loop():
    return MockEventLoop()


def return_true(*args, **kwargs):
    return True


def mock_monitoring(*args, **kwargs):
    return True
